(function ($, Drupal, drupalSettings) {
  'use strict';

  /*
   * TestCSS function.
   * @param prop
   * @returns {boolean}
   */
  function testCSS(prop) {
    return prop in document.documentElement.style;
  }

  /**
   * Function to get browser version.
   * @return {number[]}
   *   Version number.
   */
  function getArtisBrowserVersion() {
    const uMatch = navigator.userAgent.match(/ArtisBrowser\/(.*)$/);

    let ffVersion = '';
    if (uMatch && uMatch.length > 1) {
      ffVersion = [uMatch[1].split(' ')[0]];
    }
    const AC = [0, 0];
    let ix1 = 0;
    let ix2 = 0;
    if (ffVersion.length > 0) {
      ffVersion = String(ffVersion);
      ix1 = ffVersion.indexOf('.');
      ix2 = ffVersion.indexOf('.', ix1 + 1);
      if (ix1 !== -1 && ix2 !== -1) {
        AC[0] = parseInt(ffVersion.slice(0, ix1), 10);
        AC[1] = parseInt(ffVersion.slice(ix1 + 1, ix2), 10);
      }
    }
    return AC;
  }

  /**
   * Compares two software version numbers (e.g. "1.7.1")
   *
   * @param {string} v1
   *   Version V1.
   * @param {string} v2
   *  Version V2.
   * @param {array} options
   *   Options.
   * @return {number}
   * <ul>
   *    <li>0 if the versions are equal</li>
   *    <li>a negative integer iff v1 < v2</li>
   *    <li>a positive integer iff v1 > v2</li>
   *    <li>NaN if either version string is in the wrong format</li>
   * </ul>
   */
  function versionCompare(v1, v2, options) {
    const lexicographical = options && options.lexicographical;
    const zeroExtend = options && options.zeroExtend;

    let v1parts = v1.split('.');
    let v2parts = v2.split('.');

    function isValidPart(x) {
      return (lexicographical ? /^\d+[A-Za-z]*$/ : /^\d+$/).test(x);
    }

    if (!v1parts.every(isValidPart) || !v2parts.every(isValidPart)) {
      return NaN;
    }

    if (zeroExtend) {
      while (v1parts.length < v2parts.length) {
        v1parts.push('0');
      }
      while (v2parts.length < v1parts.length) {
        v2parts.push('0');
      }
    }

    if (!lexicographical) {
      v1parts = v1parts.map(Number);
      v2parts = v2parts.map(Number);
    }

    for (let i = 0; i < v1parts.length; ++i) {
      if (v2parts.length === i) {
        return 1;
      }
      if (v1parts[i] === v2parts[i]) {
        i += 1;
      }
      else if (v1parts[i] > v2parts[i]) {
        return 1;
      }
      else {
        return -1;
      }
    }

    if (v1parts.length !== v2parts.length) {
      return -1;
    }
    return 0;
  }

  /**
   *  Escape Html function.
   *
   * @param {string} unsafe
   *   Params.
   * @return {*}
   *   Safe string.
   */
  function escapeHtml(unsafe) {
    return unsafe
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;');
  }

  /**
   * The copysafe-insert functions.
   *
   * @param {string} szDocName
   *   SzDocName.
   * @param {string} wpcspPluginUrl
   *  WpcspPluginUrl.
   * @param {string} output
   * Output.
   * @param {boolean} mBpDebugging
   * MBpDebugging.
   * @param {string} mSzPlugin
   *  MSzPlugin.
   * @param {string} mBpWidth
   * MBpWidth.
   * @param {string} mBpHeight
   * MBpHeight.
   * @param {string} mSzImageFolder
   * MSzImageFolder.
   * @param {string} mSzClassName
   *   MSzClassName.
   * @param {string} mBpPrintsAllowed
   * MBpPrintsAllowed.
   * @param {string} mBpPrintAnywhere
   * MBpPrintAnywhere.
   * @param {string} mBpAllowCapture
   * MBpAllowCapture.
   * @param {string} mBpAllowRemote
   * MBpAllowRemote.
   * @param {string} mBpLanguage
   * MBpLanguage.
   * @param {string} mBpBackground
   * MBpBackground.
   */
  function insertcopysafePdfProtection(
    szDocName,
    wpcspPluginUrl,
    output,
    mBpDebugging,
    mSzPlugin,
    mBpWidth,
    mBpHeight,
    mSzImageFolder,
    mSzClassName,
    mBpPrintsAllowed,
    mBpPrintAnywhere,
    mBpAllowCapture,
    mBpAllowRemote,
    mBpLanguage,
    mBpBackground
  ) {
    const textarea = `ta${output}`;
    const string = `<param name='Document' value='${mSzImageFolder}${mSzClassName}' /><param name='PrintsAllowed' value='${mBpPrintsAllowed}' /><param name='PrintAnywhere' value='${mBpPrintAnywhere}' /><param name='AllowCapture' value='${mBpAllowCapture}' /><param name='AllowRemote' value='${mBpAllowRemote}' /><param name='Language' value='${mBpLanguage}' /><param name='Background' value='${mBpBackground}' />`;
    let szObjectInsert = '';
    if (mBpDebugging === true) {
      jQuery('#' + output).append("<textarea rows='27' cols='80' id= '" + textarea + "'>");
      if (mSzPlugin === 'DLL') {
        szObjectInsert = "type='application/x-artistscope-pdfreader5' codebase='" + wpcspPluginUrl + "download.html'";
        jQuery('#' + textarea).append(escapeHtml('<ob' + 'ject ' + szObjectInsert + 'width=\'' + mBpWidth + '\' height=\'' + mBpHeight + '\'>' + string + '</ob' + 'ject />'));
      }
      jQuery(`#${output}`).append('</textarea>');
    }
    else if (mSzPlugin === 'DLL') {
      szObjectInsert = "type='application/x-artistscope-pdfreader5' codebase='" + wpcspPluginUrl + "download.html'";
      jQuery('#' + output).append('<ob' + 'ject ' + szObjectInsert + ' width=\'' + mBpWidth + '\' height=\'' + mBpHeight + '\'>' + string + '</ob' + 'ject />');
    }
  }

  Drupal.behaviors.copysafe_pdf_protection_embed = {
    attach() {
      for (let val in drupalSettings.copysafe_pdf_protection.embed_options) {
        const wpcspPluginUrl = drupalSettings.copysafe_pdf_protection.embed_options[val]['plugin_url'];
        let mBpDebugging = false;
        const mSzMode = drupalSettings.copysafe_pdf_protection.embed_options[val]['mode'];
        const mSzClassName = drupalSettings.copysafe_pdf_protection.embed_options[val]['name'];
        const mSzImageFolder = drupalSettings.copysafe_pdf_protection.embed_options[val]['upload_url'];
        const mBpChrome = drupalSettings.copysafe_pdf_protection.embed_options[val]['chrome'];
        const mBpFx = drupalSettings.copysafe_pdf_protection.embed_options[val]['firefox'];
        const mBpASPS = drupalSettings.copysafe_pdf_protection.embed_options[val]['asps'];
        const mMinVersionASPS = drupalSettings.copysafe_pdf_protection.embed_options[val]['artisbrowser_min_version'];
        const mBpHeight = drupalSettings.copysafe_pdf_protection.embed_options[val]['bgheight'];
        const mBpWidth = drupalSettings.copysafe_pdf_protection.embed_options[val]['bgwidth'];
        const mBpBackground = drupalSettings.copysafe_pdf_protection.embed_options[val]['background'];
        const mBpPrintsAllowed = drupalSettings.copysafe_pdf_protection.embed_options[val]['prints_allowed'];
        const mBpPrintAnywhere = drupalSettings.copysafe_pdf_protection.embed_options[val]['print_anywhere'];
        const mBpAllowCapture = drupalSettings.copysafe_pdf_protection.embed_options[val]['allow_capture'];
        const mBpLanguage = drupalSettings.copysafe_pdf_protection.embed_options[val]['language'];
        const mBpAllowRemote = drupalSettings.copysafe_pdf_protection.embed_options[val]['allow_remote'];
        const mSzLocation = document.location.href.replace(/&/g, '%26');
        const mSzDownloadNo = `${wpcspPluginUrl}download_no.html`;
        const mSzDownload = `${wpcspPluginUrl}download.html?ref=${mSzLocation}`;
        const mSzAgent = navigator.userAgent.toLowerCase();
        let mSzPlugin = '';
        const mBWindows = mSzAgent.indexOf('windows nt') !== -1;
        const mBFx3 =
          mSzAgent.indexOf('firefox/3.') !== -1 &&
          mSzAgent.indexOf('flock') === -1 &&
          mSzAgent.indexOf('navigator') === -1;
        const mBFx4 =
          mSzAgent.indexOf('firefox/4.') !== -1 &&
          mSzAgent.indexOf('flock') === -1 &&
          mSzAgent.indexOf('navigator') === -1;
        const mBFirefox =
          mSzAgent.indexOf('firefox') !== -1 &&
          testCSS('MozBoxSizing') &&
          !mBFx3 &&
          !mBFx4 &&
          mBpFx;
        const mBChrome = mSzAgent.indexOf('chrome') !== -1 && mBpChrome;
        const mBASPS =
          (mSzAgent.indexOf('artisreader') !== -1 ||
            mSzAgent.indexOf('artisbrowser') !== -1) &&
          mBpASPS;
        const mBNetscape = mBChrome || mBFirefox || mBASPS;

        if (mBWindows && mBNetscape) {
          mSzPlugin = 'DLL';
        }

        if (mSzMode === 'debug') {
          mBpDebugging = true;
        }

        // In case it's windows and browser not artisBrowser.
        if (mBWindows && !mBASPS && !mBpDebugging) {
          window.location = unescape(mSzDownload);
          document.MM_returnValue = false;
        }

        // Not windows.
        if (!mBWindows) {
          window.location = unescape(mSzDownloadNo);
          document.MM_returnValue = false;
        }

        // Version check.
        if (mBWindows && mBASPS) {
          // Get the artisBrowser version.
          const arVersion = getArtisBrowserVersion();
          const szNumeric = arVersion[0] + '.' + arVersion[1];
          if (versionCompare(szNumeric, mMinVersionASPS) < 0) {
            window.location = unescape(mSzDownload);
            document.MM_returnValue = false;
          }
        }

        const output = drupalSettings.copysafe_pdf_protection.embed_options[val]['outputid'];

        if (mSzMode === 'licensed' || mSzMode === 'debug') {
          insertcopysafePdfProtection(
            mSzClassName,
            wpcspPluginUrl,
            output,
            mBpDebugging,
            mSzPlugin,
            mBpWidth,
            mBpHeight,
            mSzImageFolder,
            mSzClassName,
            mBpPrintsAllowed,
            mBpPrintAnywhere,
            mBpAllowCapture,
            mBpAllowRemote,
            mBpLanguage,
            mBpBackground
          );
        }
        else {
          document.getElementById(
            output
          ).innerHTML = `<img src='${wpcspPluginUrl}images/demo_placeholder.jpg' border='0' alt='Demo mode'>`;
        }
      }
    }
  };
})(jQuery, Drupal, drupalSettings);
